#!/usr/bin/env ruby

# written by Greg Willits Feb 2010
# www.gregwillits.ws/articles/myrails

# Put myrails.rb in /usr/local/bin (or some location you prefer which won't
# require entering the full path name on your command line)

# The myrails.rb file is used like this where [command] is optional:
# 	myrails.rb project_name [--start]
# 	myrails.rb project_name --stop
# 	myrails.rb project_name --stop_all

# A --start is equivalent to a restart, meaning 
# if the rails port is already running, that port will be stopped, then started again

# A config file named myrails.conf typically stored in /etc is used 
# to define project parameters for myrails.rb as a YAML hash of hashes config file.
# Redefine $projects below to use a different location.

# Each project definition require three elements:
# 	:project_name:   a label to use in the command line
#   	:app_root:   the path to the folder containing the rails application
#   	:port:       a port number to run the rails app on

require 'open3'
require 'optparse'
require 'rubygems'
require 'yaml'

#-----------------------------------------------------------
#: projects

$projects = YAML.load_file('/etc/myrails.conf')


#-----------------------------------------------------------
#: paths

def app_root
	"#{$projects[$project_name][:app_root]}"
end

def log_root
	"#{$projects[$project_name][:app_root]}/log"
end

def dev_log
	"#{log_root}/development.log"
end

def mongrel_log
	"#{log_root}/mongrel.log"
end

def mongrel_pid
	"#{log_root}/mongrel.pid"
end

def mongrel_port
	"#{$projects[$project_name][:port]}"
end

#------------------------------------------------------------
def stop_rails
	puts
	puts "Stopping rails..."
	if FileTest.exists?(mongrel_pid)
		result = system("mongrel_rails stop -c #{app_root}")
	end
	puts "Clearing logs..."
	result = system("rm -f #{mongrel_log}")
	result = system("rm -f #{dev_log}")
	puts "Removing stray pid..."
	result = system("rm -f #{mongrel_pid}")
	puts "Done"
	puts
end

#------------------------------------------------------------
def start_rails
	puts "Starting rails (#{mongrel_port})..."
	result = system("mongrel_rails start -d -c #{app_root} -p #{mongrel_port}")
	puts "Done"
	puts
end

#-----------------------------------------------------------
#: command line options

cli_args = {}

opts = OptionParser.new
opts.on("--start", "starts or restarts a site") 	{|value| cli_args[:action] = :start}
opts.on("--stop", "stops a site") 					{|value| cli_args[:action] = :stop}
opts.on("--stop_all", "stops all sites")			{|value| cli_args[:action] = :stop_all}
extras = opts.parse!(ARGV)

if extras.size == 0
	puts
	puts "Please specify one of these projects:"
	puts "#{$projects.keys.join(', ')}"
	puts
	exit
end

$project_name 	= extras[0].to_sym
$action 		= :start
$action 		= cli_args[:action] if cli_args[:action]

case $action
	when :start
		stop_rails
		start_rails
	when :stop
		stop_rails
	when :stop_all
		$projects.each_key do |project|
			$project_name = project
			puts "--- #{$project_name} ---"
			stop_rails
		end
end

